==> jimmer-sql-0.0.9-sources.jar: org/babyfish/jimmer/sql/fetcher/impl/FetcherContext.java <==
package org.babyfish.jimmer.sql.fetcher.impl;

import org.babyfish.jimmer.runtime.DraftSpi;
import org.babyfish.jimmer.sql.SqlClient;
import org.babyfish.jimmer.sql.fetcher.Fetcher;
import org.babyfish.jimmer.sql.fetcher.Field;

import java.sql.Connection;
import java.util.Collection;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.Map;
import java.util.function.BiConsumer;

class FetcherContext {

    private static final ThreadLocal<FetcherContext> FETCHER_CONTEXT_LOCAL = new ThreadLocal<>();

    private SqlClient sqlClient;

    private Connection con;

    private DataCache cache = new DataCache();

    private Map<Field, FetcherTask> taskMap = new LinkedHashMap<>();

    public static void using(
            SqlClient sqlClient,
            Connection con,
            BiConsumer<FetcherContext, Boolean> block
    ) {
        FetcherContext ctx = FETCHER_CONTEXT_LOCAL.get();
        if (ctx != null) {
            block.accept(ctx, false);
        } else {
            ctx = new FetcherContext(sqlClient, con);
            FETCHER_CONTEXT_LOCAL.set(ctx);
            try {
                block.accept(ctx, true);
            } finally {
                FETCHER_CONTEXT_LOCAL.remove();
            }
        }
    }

    private FetcherContext(SqlClient sqlClient, Connection con) {
        this.sqlClient = sqlClient;
        this.con = con;
    }

    public void add(Fetcher<?> fetcher, DraftSpi draft) {
        for (Field field : fetcher.getFieldMap().values()) {
            if (!field.isSimpleField()) {
                FetcherTask task = taskMap.computeIfAbsent(field, it ->
                        new FetcherTask(
                                cache,
                                sqlClient,
                                con,
                                field
                        )
                );
                task.add(draft);
            }
        }
    }

    public void addAll(Fetcher<?> fetcher, Collection<DraftSpi> drafts) {
        for (DraftSpi draft : drafts) {
            add(fetcher, draft);
        }
    }

    public void execute() {
        while (!taskMap.isEmpty()) {
            Iterator<Map.Entry<Field, FetcherTask>> itr = taskMap.entrySet().iterator();
            Map.Entry<Field, FetcherTask> e = itr.next();
            if (e.getValue().execute()) {
                taskMap.remove(e.getKey());
            }
        }
    }
}

==> jimmer-sql-0.0.33-sources.jar: org/babyfish/jimmer/sql/fetcher/impl/FetcherContext.java <==
package org.babyfish.jimmer.sql.fetcher.impl;

import org.babyfish.jimmer.runtime.DraftSpi;
import org.babyfish.jimmer.sql.SqlClient;
import org.babyfish.jimmer.sql.fetcher.Fetcher;
import org.babyfish.jimmer.sql.fetcher.Field;
import org.babyfish.jimmer.sql.fetcher.RecursionStrategy;

import java.sql.Connection;
import java.util.Collection;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.Map;
import java.util.function.BiConsumer;

class FetcherContext {

    private static final ThreadLocal<FetcherContext> FETCHER_CONTEXT_LOCAL = new ThreadLocal<>();

    private SqlClient sqlClient;

    private Connection con;

    private DataCache cache = new DataCache();

    private Map<Field, FetcherTask> taskMap = new LinkedHashMap<>();

    public static void using(
            SqlClient sqlClient,
            Connection con,
            BiConsumer<FetcherContext, Boolean> block
    ) {
        FetcherContext ctx = FETCHER_CONTEXT_LOCAL.get();
        if (ctx != null) {
            block.accept(ctx, false);
        } else {
            ctx = new FetcherContext(sqlClient, con);
            FETCHER_CONTEXT_LOCAL.set(ctx);
            try {
                block.accept(ctx, true);
            } finally {
                FETCHER_CONTEXT_LOCAL.remove();
            }
        }
    }

    private FetcherContext(SqlClient sqlClient, Connection con) {
        this.sqlClient = sqlClient;
        this.con = con;
    }

    @SuppressWarnings("unchecked")
    public void add(Fetcher<?> fetcher, DraftSpi draft) {
        for (Field field : fetcher.getFieldMap().values()) {
            if (!field.isSimpleField()) {
                RecursionStrategy<?> recursionStrategy = field.getRecursionStrategy();
                if (recursionStrategy != null &&
                        !((RecursionStrategy<Object>)recursionStrategy).isRecursive(
                                new RecursionStrategy.Args<>(draft, 0)
                        )
                ) {
                    return;
                }
                FetcherTask task = taskMap.computeIfAbsent(field, it ->
                        new FetcherTask(
                                cache,
                                sqlClient,
                                con,
                                field
                        )
                );
                task.add(draft);
            }
        }
    }

    public void addAll(Fetcher<?> fetcher, Collection<DraftSpi> drafts) {
        for (DraftSpi draft : drafts) {
            add(fetcher, draft);
        }
    }

    public void execute() {
        while (!taskMap.isEmpty()) {
            Iterator<Map.Entry<Field, FetcherTask>> itr = taskMap.entrySet().iterator();
            Map.Entry<Field, FetcherTask> e = itr.next();
            if (e.getValue().execute()) {
                taskMap.remove(e.getKey());
            }
        }
    }
}
